// Yoonha Choi                                   
// Date: Aug 21, 2009
//
//
//
// 1. Chi-Square test 
// 2. Corrected Chi-Square test
//
//

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>

void chi_square_tests(int *y,double **Kmat, int *Mrk, double N_Kmat_N, double N_Kmat_C, double C_Kmat_C, int n_ind, FILE *outfile);

int main(int argc, char *argv[]){
  if(argc != 7){
    printf("USAGE:./cor_chi <phenotype_file> <K_mat> <SNP_file> <output file> <#individuals> <#markers>\n");
    exit(1);
  }

  FILE *datafile, *Kmatfile, *SNPfile, *SNPindex, *outfile;
  int n_ind, n_mrk;
  int *y, *Mrk, *mrk_index;
  int i, j, k, n_mrk_index,size_snp_index; 

  double **Kmat, Kmat_ij, N_Kmat_N=0, N_Kmat_C= 0, C_Kmat_C = 0;

  if(!(datafile=fopen(argv[1],"r")))
    puts("Error opening phenotype file\n");

  if(!(Kmatfile=fopen(argv[2],"r")))
    puts("Error opening K_mat file\n");
  
  if(!(SNPfile=fopen(argv[3],"r")))
    puts("Error opening SNP file\n");

//  if(!(SNPindex=fopen(argv[4],"r")))
//    puts("Error opening SNP index\n");

  if((outfile=fopen(argv[4],"w"))==NULL)
    printf("Error creating output file\n");
  
  fprintf(outfile,"N AF_all AF_case Chi_square Cor_Chi_square\n");

  n_ind = atoi(argv[5]);
  n_mrk = atoi(argv[6]);
  size_snp_index = n_ind;
  
  /* Load Data */
  y = (int *)malloc((size_t)(n_ind)*sizeof(int));
  Kmat = (double **)malloc((size_t)(n_ind)*sizeof(double *));
  mrk_index = (int *)malloc((size_t)(size_snp_index)*sizeof(int));
  
  for(i=0; i<n_ind;i++){
    fscanf(datafile,"%d",&y[i]);
    Kmat[i] = (double *)malloc((size_t)(n_ind)*sizeof(double));
    for(j=0; j<n_ind;j++){
      fscanf(Kmatfile,"%lf",&Kmat_ij);
      Kmat[i][j] = Kmat_ij;     
    }    
  }
  close(Kmatfile);
  close(datafile);

  for(i=0;i<n_ind;i++){
    for(j=0;j<n_ind;j++){
      N_Kmat_N = N_Kmat_N + Kmat[i][j];
      if(y[j]==1){
	N_Kmat_C = N_Kmat_C + Kmat[i][j];
	if(y[i]==1){
	  C_Kmat_C = C_Kmat_C + Kmat[i][j];
	}
      }
    }
  }

  //  n_mrk_index=0;
  // while(!feof(SNPindex)){
  //   fscanf(SNPindex,"%d",&mrk_index[n_mrk_index]); 
  //   n_mrk_index = n_mrk_index + 1;
  //  }
  // close(SNPindex);
  n_mrk_index = size_snp_index;


  /* Chi-Square and Corrected Chi-Square Test */

  char snp_tmp[n_mrk_index], mrk_tmp;
  Mrk= (int *)malloc((size_t)(n_ind)*sizeof(int));

  for(i=0;i<n_mrk;i++){
    
    fgets(snp_tmp,(n_mrk_index+1),SNPfile);
    if((int)strlen(snp_tmp) != n_ind){
      fgets(snp_tmp,(n_mrk_index+1),SNPfile);
    }
    //k=0;
    //    printf("%s\n",snp_tmp);

    for(j=0;j< (int) strlen(snp_tmp);j++){
      mrk_tmp = snp_tmp[j];
      //     if(mrk_index[j]==1){
      Mrk[j] = atoi(&mrk_tmp);
      //k = k+1;
      //      }
    }
    chi_square_tests(y, Kmat, Mrk, N_Kmat_N, N_Kmat_C,C_Kmat_C, n_ind, outfile);
  } 
  
  close(outfile);
}

void chi_square_tests(int *y,double **Kmat, int *Mrk, double N_Kmat_N, double N_Kmat_C,double C_Kmat_C, int n_ind, FILE *outfile){

  int N_case, N_ctrl, N;
  int i, j, k, k1, k2;
  double X_bar,XC_bar;
  double numerator,W_chi,W_cor_chi;

  // Count the number of individuals without missing SNP data
  // Compute X_bar and XC_bar
  N_case = 0;
  N_ctrl = 0;
  X_bar = 0;
  XC_bar = 0;

  for(i=0;i<n_ind;i++){
    if(Mrk[i] != 9){
      //      printf("%d ",Mrk[i]);
      X_bar = X_bar + Mrk[i];
      if(y[i] == 1){
	XC_bar = XC_bar + Mrk[i];
	N_case++;
      }else if(y[i] == 0){
	N_ctrl++;
      }else{
	fprintf(outfile, "ERROR\n");
      }
    }
  }
  //  printf("\n");

  N=N_case+N_ctrl;
  
  if(N != 0 && N_case !=0){
    X_bar = X_bar/(double)(N*2);
    XC_bar = XC_bar/(double)(N_case*2);
  }
   
  // Compute test statistics 
  if( X_bar == 0 || X_bar ==1 || XC_bar == 0 || XC_bar ==1){
     fprintf(outfile,"0 NA NA NA NA\n");
  }else{

    for(i=0;i<n_ind;i++){
      for(j=0;j<n_ind;j++){

	if(Mrk[i] ==9 ||  Mrk[j] ==9 ){
	  N_Kmat_N = N_Kmat_N - Kmat[i][j];
	
	  if(y[j] == 1){
	    N_Kmat_C = N_Kmat_C - Kmat[i][j];
	  }

	  if(y[i] == 1 && y[j] == 1){
	    C_Kmat_C = C_Kmat_C - Kmat[i][j];
	  }	  
	}
      } //The end of for(j=0;j<n_ind;j++){
    } //The end of for(i=0;j<n_ind;i++){

    //    printf("%f %f %f \n",N_Kmat_N,N_Kmat_C,C_Kmat_C);

    numerator = (XC_bar-X_bar)*(XC_bar-X_bar);
    
    W_chi = (double) numerator*2/X_bar/(1-X_bar)/((1/(double)N_case) - (1/(double)N));
    W_cor_chi =(double) numerator*2/X_bar/(1-X_bar)/((1/(double)N_case/(double)N_case)*C_Kmat_C -2*(1/(double)N/(double)N_case)*N_Kmat_C + (1/(double)N/(double)N)*N_Kmat_N);
   
    fprintf(outfile,"%d %f %f %f  %f\n",N,X_bar,XC_bar,W_chi,W_cor_chi);   
  } // The end of }else{
}

