#!/usr/bin/perl -w
#
# Yoonha Choi and Mark Marchani   Date: July 12, 2007  Updated: Mar 28, 2010
#
# Purpose:
#  Compute kinship coefficient and inbreeding coefficients from estimated k-statistics  or ibd coefficients
#
# Usage:
#  ./kinship.pl <Infile> <Outfile>
#  e.g.) ./kinship.pl comb.out comb.kin
#
# Input file: the result file from "kstat.pl", "kstat.c" or "ibd_d9.c"
#
# Output file:
#  columns - sub_1 sub_2 kinship
#
#####

########## ARGUEMENT ################

if(@ARGV<2){
  print "\nUSAGE: ./kinship.pl <Infile> <Outfile>\n";
  print "EXAMPLE: ./kinship.pl comb.out comb.kin\n";
  exit(0);
}

($infile,$outfile) = @ARGV;

############ MAIN #################

open(IN,"<$infile")or die "Couldn't open $infile \n";
open(OUT,">$outfile")or die "Couldn't open $outfile \n";
printf OUT ("%10s %10s %10s\n","sub_1","sub_2","kin");

my $header = <IN>;
while(<IN>){

  my @line=split(" ",$_);
  my $line_length = @line;
  unless ($line_length == 5 || $line_length == 7 || $line_length == 11)
    {
     die "Wrong number of arguments in the line read.  There are $line_length items in the line.\n";
     }

  if ($line_length == 5 || $line_length == 7) 
    { $sub1 = $line[0];
      $sub2 = $line[1];
      $k1   = $line[3];
      $k2   = $line[4];

      $kin = 1/2*$k2 + 1/4*$k1;
      printf OUT ("%10s %10s %10s\n",$sub1,$sub2,$kin);
      } else        
        { $sub1 = $line[0];
          $sub2 = $line[1];
          $d1   = $line[2];
	  $d2   = $line[3];
          $d3   = $line[4];
	  $d4   = $line[5];
          $d5   = $line[6];
	  $d6   = $line[7];
          $d7   = $line[8];
          $d8   = $line[9];

          $kin = $d1 + 1/2 * ($d3 + $d5 + $d7) + 1/4 * $d8;
          $ibd1 = $d1 + $d2 + $d3 + $d4;
	  $ibd2 = $d1 + $d2 + $d5 + $d6;
	  printf OUT ("%10s %10s %10s\n",$sub1,$sub2,$kin);

	  if(!exists($id_index{$sub1})){
	    push(@id_array,$sub1);
	    $id_index{$sub1}=1;
	    $ibd{$sub1}= $ibd1;
	  }else{
	    $ibd{$sub1} = $ibd{$sub1} + $ibd1;
	    $id_index{$sub1} = $id_index{$sub1} +1;
	  }
	  if(!exists($id_index{$sub2})){
	    push(@id_array,$sub2);
	    $id_index{$sub2}=1;
	    $ibd{$sub2} = $ibd2;
	  }else{
	    $ibd{$sub2} = $ibd{$sub2} + $ibd2;
	    $id_index{$sub2} = $id_index{$sub2} +1;
	  }
          
	}
}
close(IN);

foreach $id (@id_array){
  $ibd_tmp = $ibd{$id}/$id_index{$id};
  print OUT "$id $id $ibd_tmp\n";
}

close(OUT);

