#!/usr/bin/perl -w

# Yoonha Choi   Last Modified:   Created: 07/05/07
#
# Purpose
# -------
# Create allele counts for genotype data.
# Input file is rectangular with a header, id in first col, followed by marker data
# (e.g. A/A or -0-).
#
# USAGE:
#  ./allele_freq.pl <Infile> <Outfile>
#
#
# Output Files
# -----------
# First output file is named by "input file name.frq":
# - Marker name (Capitalize 'd' and 's')
# - Allele value
# - Allele count
# - Allele frequency
#

########## ARGUEMENTS ##########
if (@ARGV<1){
  print "\nUSAGE: ./allele_freq.pl <infile> <outfile>\n";
  print "EXAMPLE: ./allele_freq.pl set0.dat set0.frq\n";
  exit(0);
}

($infile,$outfile_frq) = @ARGV;

############# MAIN ROUTINE #################
infile();
frqfile();


############# SUB ROUTINES #################

# Load marker names, alleles
# --------------------------

sub infile {
  open (IN, "<$infile") or die "Couldn't open $infile:$!\n";
  my $header = <IN>;                        # Read in header
#  $header =~ s/d/D/g;                       # Capitalize s and d in marker names
#  $header =~ s/s/S/g;
  (my $sub,@mrk_name) = split (' ',$header);     # Array of marker names

  while (<IN>) {
    (my $id, @mrk_data) = split(' ',$_);
    foreach my $i (0..$#mrk_data){
      if($mrk_data[$i] ne "-0-") {  # Match non-missing mrk data
	($allele1, $allele2) = split("/",$mrk_data[$i]);
	push( @{$all[$i]}, $allele1, $allele2);          # Array of arrays of alleles at each mrk

	my $repeated1 = already_listed($i, $allele1);
	if ($repeated1 eq "no") {
	  push(@{$list[$i]}, $allele1);             # Array of arrays of unique alleles
	}
	my $repeated2 = already_listed($i, $allele2);
	if ($repeated2 eq "no") {
	  push (@{$list[$i]}, $allele2);
	}
      }
    }
  }
  close (IN);
}

# Function to check whether the item is already listed or not
# -----------------------------------------------------------
sub already_listed {
  my $mrki = shift;
  my $allele = shift;
  my $listed = "no";
  foreach (@{$list[$mrki]}) {
    if ($allele eq $_){
      $listed = "yes";
      last;
    }else{ $listed = "no"; }
  }
  return $listed;
}

# Output allele counts and frequencies
# ------------------------------------
sub frqfile {
  open (OUT1, ">$outfile_frq") or die "Couldn't open $outfile_frq: $!\n";
  print OUT1 "        marker     all   count   freq\n";
  foreach my $i (0..$#mrk_name){
    @{$sorted[$i]} = sort (@{$list[$i]});       # Sort alleles numerically
    foreach my $allele (@{$sorted[$i]}) {
      $count[$i]{$allele} = 0;                              # Count each allele at each mrk
      foreach my $observed (@{$all[$i]}) {
	if ($observed eq $allele) { $count[$i]{$allele}++;}
      }

      ### Array of hashes of allele freqs at each mrk, keyed by allele values
      $freq[$i]{$allele} = $count[$i]{$allele}/@{$all[$i]}; 
      printf OUT1 "%13s %5s %5s %6.4f\n", $mrk_name[$i],$allele,$count[$i]{$allele},$freq[$i]{$allele};
    }
  }
  close(OUT1);
}
