#!/usr/bin/perl -w
#
# Yoonha Choi     Date: Mar. 26, 2010
#
# Usage: ./kin_to_kmat.pl <ID file> <Kinship file>
#    e.g)./kin_to_kmat.pl sample.id sample.kin
#
# Input files:
#    sample.id
#    sample.kin
# 
# Output files:
#    k_matrix.out
#
###

##### Arguements #####
if(@ARGV < 2){
  print ("\nUsage: ./kin_to_kmat.pl <ID file> <Kinship file>\n");
  print ("EXAMPLE: ./kin_to_kmat.pl sample.id sample.kin\n");
  exit(0);
}
($id_file, $kin_file) = @ARGV;

##### Input files and Output file #####
open(KMAT,">k_matrix.out") or die "Couldn't open k_matrix.out: $!\n";
open(ID,"<$id_file") or die "Couldn't open $id_file\n";

######### Load IDs ########
while(<ID>){
    ($id,my @tmp) = split(" ",$_);
    push(@id_array,$id);
    $id_index{$id}=1;
}
close(ID);

############### Load kinship and inbreeding coefficients ############
open(KIN,"<$kin_file") or die "Couldn't open $kin_file: $!\n";
my $header = <KIN>;
while(<KIN>){
    ($id1,$id2,$kin)=split(" ",$_);

    if(exists($id_index{$id1}) && exists($id_index{$id2})){
      $keys = "$id1/$id2";
      push(@kin_list_tmp,$keys,$kin);
    }
}
%kin_list = @kin_list_tmp;
close(KIN);

######################### K MATRIX ##############################
foreach $i (0..$#id_array){
    foreach $j ($i..$#id_array){
	$id1 = $id_array[$i];
	$id2 = $id_array[$j];
	$keys = "$id1/$id2";
	$keys2 = "$id2/$id1";
	if($i eq $j){
	    if(exists $kin_list{$keys}){
		$inbreed=$kin_list{$keys};
	    }elsif(exists $kin_list{$keys2}){
		$inbreed=$kin_list{$keys2};
	    }else{$inbreed = 0;}
	    push(@{$kmat[$i]}, 1+$inbreed);
	}
	elsif($i <$j){
	    if(exists $kin_list{$keys}){
		$kinship=$kin_list{$keys};
	    }elsif(exists $kin_list{$keys2}){
		$kinship=$kin_list{$keys2};
	    }else{
		$kinship = 0;
	    }
	    $kin = $kinship *2;

	    push(@{$kmat[$i]},$kin);
	    push(@{$kmat[$j]},$kin);
	}
    }
}

############## PRINT OUT K MATRIX #################
@col = @{$kmat[0]};
$ncol=$#col;

foreach $a (0..$ncol){
    @k_tmp = @{$kmat[$a]};
    print KMAT "@k_tmp\n";
}

close(KMAT);

