#!/usr/bin/perl -w
#
# Yoonha Choi     Date: Mar. 26, 2010
#
# Usage: ./kstat_ibd_to_kmat.pl <ID file> <kstat or ibd file>
#    e.g)./kstaT_ibd_to_kmat.pl sample.id sample_kstat.out
#
# Input files:
#    sample.id
#    sample_kstat.out
# 
# Output files:
#    k_matrix.out
#
###

##### Arguements #####
if(@ARGV < 2){
  print ("\nUsage: ./kstat_ibd_to_kmat.pl <ID file> <kstat or ibd file>\n");
  print ("EXAMPLE: ./kstat_ibd_to_kmat.pl sample.id sample_kstat.out\n");
  exit(0);
}
($id_file, $ibd_file) = @ARGV;

##### Input files and Output file #####
open(KMAT,">k_matrix.out") or die "Couldn't open k_matrix.out: $!\n";
open(ID,"<$id_file") or die "Couldn't open $id_file\n";

######### Load IDs ########
while(<ID>){
    ($id,my @tmp) = split(" ",$_);
    push(@id_array,$id);
    $id_index{$id}=1;
}
close(ID);

############### Load kinship and inbreeding coefficients ############
open(IBD,"<$ibd_file") or die "Couldn't open $ibd_file: $!\n";
my $header = <IBD>;
while(<IBD>){
  ($id1,$id2,@ibd)=split(" ",$_);
  unless ($#ibd == 2 || $#ibd == 4 || $#ibd == 8){
    die "Wrong number of arguments in the line read\n";
  }
  
  if(exists($id_index{$id1}) && exists($id_index{$id2})){
    $keys = "$id1/$id2";
    
    if($#ibd == 2 || $#ibd == 4){
      $kin = 1/2*$ibd[2] + 1/4*$ibd[1];
      $kin_list{$keys} = $kin;
    }else{
      $keys_ibd1 = "$id1/$id1";
      $keys_ibd2 = "$id2/$id2";
      $kin = $ibd[0] + 0.5*($ibd[2] + $ibd[4] + $ibd[6]) + 0.25*$ibd[7];
      $ibd1 = $ibd[0] + $ibd[1] + $ibd[2] + $ibd[3];
      $ibd2 = $ibd[0] + $ibd[1] + $ibd[4] + $ibd[5];
      $kin_list{$keys} = $kin;
	
      if(exists($kin_list{$keys_ibd1})){
	$kin_list{$keys_ibd1} = $kin_list{$keys_ibd1} + $ibd1; 
	$ibd_index{$keys_ibd1} = $ibd_index{$keys_ibd1} +1;
      }else{
	$kin_list{$keys_ibd1} = $ibd1;
	$ibd_index{$keys_ibd1} = 1;
      }
      
      if(exists($kin_list{$keys_ibd2})){
	$kin_list{$keys_ibd2} = $kin_list{$keys_ibd2} + $ibd2; 
	$ibd_index{$keys_ibd2} = $ibd_index{$keys_ibd2} +1;
      }else{
	$kin_list{$keys_ibd2} = $ibd2;
	$ibd_index{$keys_ibd2} = 1;
      }
    }
  }
}
close(IBD);

if($#ibd == 8){
  foreach $id (@id_array){
    $keys="$id/$id";
    $kin_list{$keys} = $kin_list{$keys}/$ibd_index{$keys};
  }
}

######################### K MATRIX ##############################
foreach $i (0..$#id_array){
    foreach $j ($i..$#id_array){
	$id1 = $id_array[$i];
	$id2 = $id_array[$j];
	$keys = "$id1/$id2";
	$keys2 = "$id2/$id1";
	if($i eq $j){
	    if(exists $kin_list{$keys}){
		$inbreed=$kin_list{$keys};
	    }elsif(exists $kin_list{$keys2}){
		$inbreed=$kin_list{$keys2};
	    }else{$inbreed = 0;}
	    push(@{$kmat[$i]}, 1+$inbreed);
	}
	elsif($i <$j){
	    if(exists $kin_list{$keys}){
		$kinship=$kin_list{$keys};
	    }elsif(exists $kin_list{$keys2}){
		$kinship=$kin_list{$keys2};
	    }else{
		$kinship = 0;
	    }
	    $kin = $kinship *2;

	    push(@{$kmat[$i]},$kin);
	    push(@{$kmat[$j]},$kin);
	}
    }
}

############## PRINT OUT K MATRIX #################
@col = @{$kmat[0]};
$ncol=$#col;

foreach $a (0..$ncol){
    @k_tmp = @{$kmat[$a]};
    print KMAT "@k_tmp\n";
}

close(KMAT);
