//
// Yoonha Choi
//

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

double prob_p1(int geno1, int geno2, double p);
double prob_p2(int geno1, int geno2, double p);
double prob_p3(int geno1, int geno2, double p);
double prob_p4(int geno1, int geno2, double p);
double prob_p5(int geno1, int geno2, double p);
double prob_p6(int geno1, int geno2, double p);
double prob_p7(int geno1, int geno2, double p);
double prob_p8(int geno1, int geno2, double p);
double prob_p9(int geno1, int geno2, double p);

int main(int argc,char *argv[])
{
  if(argc != 5){
    printf("USAGE:./ibd_d9.o <input_file> <output_file> <#individuals> <#markers>\n");
    exit(1);
  }
  
  FILE *infile, *outfile; 
  int n_mrk, n_ind,m;
  double *freq;
  double *p1,*p2,*p3,*p4,*p5,*p6,*p7,*p8,*p9;

  char tmpid1[20],tmpid2[20]; //ID of a pair of individuals
  int *geno1,*geno2;          //Genotype of a pair of individuals
  int n_id1,n_id2;

  n_ind = atoi(argv[3]);
  n_mrk = atoi(argv[4]);

  if(!(infile=fopen(argv[1],"r")))
    puts("Error opening input file\n");

  if((outfile=fopen(argv[2],"w"))==NULL)
    printf("Error creating output file\n");
    
  fprintf(outfile,"ID1 ID2 d1 d2 d3 d4 d5 d6 d7 d8 d9\n");

  // Allele Frequency  
  freq = (double *)malloc((size_t)(n_mrk)*sizeof(double));
  allele_freq(n_ind,n_mrk,infile,freq);
  printf("allele freq done !\n");
  //   for(m=1;m<=n_mrk;m++){
  //    printf("%f ",freq[(m-1)]);
  //  }

  // Compute Pr(Si|delta=j), j=1,...,9

  p1 = (double *)malloc((size_t)(n_mrk)*sizeof(double));
  p2 = (double *)malloc((size_t)(n_mrk)*sizeof(double));
  p3 = (double *)malloc((size_t)(n_mrk)*sizeof(double));
  p4 = (double *)malloc((size_t)(n_mrk)*sizeof(double));
  p5 = (double *)malloc((size_t)(n_mrk)*sizeof(double));
  p6 = (double *)malloc((size_t)(n_mrk)*sizeof(double));
  p7 = (double *)malloc((size_t)(n_mrk)*sizeof(double));
  p8 = (double *)malloc((size_t)(n_mrk)*sizeof(double));
  p9 = (double *)malloc((size_t)(n_mrk)*sizeof(double));
 
  for(m=1;m<=n_mrk;m++){
    p1[(m-1)]=0;
    p2[(m-1)]=0;
    p3[(m-1)]=0;
    p4[(m-1)]=0;
    p5[(m-1)]=0;
    p6[(m-1)]=0;
    p7[(m-1)]=0;
    p8[(m-1)]=0;
    p9[(m-1)]=0;
  }

  geno1 = (int *)malloc((size_t)(n_mrk)*sizeof(int));
  geno2 = (int *)malloc((size_t)(n_mrk)*sizeof(int));

  for(n_id1 = 1; n_id1 <= (n_ind-1); n_id1++){

    // Skip the individuals before id1 
    for(n_id2 = 1; n_id2<= n_id1 ; n_id2++){
      fscanf(infile,"%s\t",tmpid1);
      for(m=1 ; m <= n_mrk ; m++){
	fscanf(infile,"%d",&geno1[(m-1)]);
      }
    }

    while(fscanf(infile,"%s\t",tmpid2)==1){  
      for(m=1 ; m <= n_mrk ; m++){
	fscanf(infile,"%d",&geno2[(m-1)]);
	p1[(m-1)]=(double)prob_p1(geno1[(m-1)],geno2[(m-1)],freq[(m-1)]);
	p2[(m-1)]=(double)prob_p2(geno1[(m-1)],geno2[(m-1)],freq[(m-1)]);
	p3[(m-1)]=(double)prob_p3(geno1[(m-1)],geno2[(m-1)],freq[(m-1)]);
	p4[(m-1)]=(double)prob_p4(geno1[(m-1)],geno2[(m-1)],freq[(m-1)]);
	p5[(m-1)]=(double)prob_p5(geno1[(m-1)],geno2[(m-1)],freq[(m-1)]);
	p6[(m-1)]=(double)prob_p6(geno1[(m-1)],geno2[(m-1)],freq[(m-1)]);
	p7[(m-1)]=(double)prob_p7(geno1[(m-1)],geno2[(m-1)],freq[(m-1)]);
	p8[(m-1)]=(double)prob_p8(geno1[(m-1)],geno2[(m-1)],freq[(m-1)]);
	p9[(m-1)]=(double)prob_p9(geno1[(m-1)],geno2[(m-1)],freq[(m-1)]);
      }
      em_algorithm(outfile,tmpid1,tmpid2,n_mrk,p1,p2,p3,p4,p5,p6,p7,p8,p9);
    }
    rewind(infile);    
  }


  close(infile);
  close(outfile);
}


/* Function for Estimating Allele Frequencies */
int allele_freq(int n_ind, int n_mrk, FILE *infile, double *freq)
{
  char tmpid[20];
  int m, allele;
  int *numerator, *denominator;
  double tmp_double;

  numerator = (int *)malloc((size_t)(n_mrk)*sizeof(int));
  denominator = (int *)malloc((size_t)(n_mrk)*sizeof(int));
  
  for(m=1; m<=n_mrk;m++){
    numerator[(m-1)]=0;
    denominator[(m-1)]=n_ind*2;
  }

  while(fscanf(infile,"%s\t",tmpid)==1){
    for(m=1 ; m <= n_mrk ; m++){
      fscanf(infile,"%d",&allele);

      if(allele == -1){
	denominator[(m-1)] -= 2;
      }else{
	numerator[(m-1)] += allele;

      }
    }
  }
  
  for(m=1; m <= n_mrk ; m++){
    tmp_double= (double)numerator[(m-1)]/denominator[(m-1)];
    freq[(m-1)] = (double)tmp_double;
    //    printf("%d %d %f\n",numerator[(m-1)],denominator[(m-1)],freq[(m-1)]);
  }

  rewind(infile);
  return 0;
}

double prob_p1(int geno1, int geno2, double p)
{
  double prob1=0;
  if(geno1==0 && geno2==0)
    prob1=(1-p);
 
  if(geno1==2 && geno2==2)
    prob1=p;

  return prob1;
}

double prob_p2(int geno1, int geno2, double p)
{
  double prob2=0;
  if(geno1==0 && geno2==0)
    prob2=(1-p)*(1-p);
 
  if((geno1==0 && geno2==2)||(geno1==2 && geno2==0))
    prob2=p*(1-p);

  if(geno1==2 && geno2==2)
    prob2=p*p;

  return prob2;
}

double prob_p3(int geno1, int geno2, double p)
{
  double prob3=0;
  if(geno1==0 && geno2==0)
    prob3=(1-p)*(1-p);
 
  if((geno1==0 && geno2==1)||(geno1==2 && geno2==1))
    prob3=p*(1-p);

  if(geno1==2 && geno2==2)
    prob3=p*p;

  return prob3;
}

double prob_p4(int geno1, int geno2, double p)
{
  double prob4=0;
  if(geno1==0 && geno2==0)
    prob4=(1-p)*(1-p)*(1-p);
 
  if(geno1==0 && geno2==1)
    prob4=2*p*(1-p)*(1-p);

  if(geno1==0 && geno2==2)
    prob4=p*p*(1-p);

  if(geno1==2 && geno2==0)
    prob4=p*(1-p)*(1-p);

  if(geno1==2 && geno2==1)
    prob4=2*p*p*(1-p);

  if(geno1==2 && geno2==2)
    prob4=p*p*p;

  return prob4;
}

double prob_p5(int geno1, int geno2, double p)
{
  double prob5=0;
  if(geno1==0 && geno2==0)
    prob5=(1-p)*(1-p);
 
  if(geno1==1 && geno2==0)
    prob5=p*(1-p);

  if(geno1==1 && geno2==2)
    prob5=p*(1-p);

  if(geno1==2 && geno2==2)
    prob5=p*p;

  return prob5;
}

double prob_p6(int geno1, int geno2, double p)
{
  double prob6=0;
  if(geno1==0 && geno2==0)
    prob6=(1-p)*(1-p)*(1-p);
 
  if(geno1==0 && geno2==2)
    prob6=p*(1-p)*(1-p);

  if(geno1==1 && geno2==0)
    prob6=2*p*(1-p)*(1-p);

  if(geno1==1 && geno2==2)
    prob6=2*p*p*(1-p);

  if(geno1==2 && geno2==0)
    prob6=p*p*(1-p);

  if(geno1==2 && geno2==2)
    prob6=p*p*p;

  return prob6;
}

double prob_p7(int geno1, int geno2, double p)
{
  double prob7=0;
  if(geno1==0 && geno2==0)
    prob7=(1-p)*(1-p);
 
  if(geno1==1 && geno2==1)
    prob7=2*p*(1-p);

  if(geno1==2 && geno2==2)
    prob7=p*p;

  return prob7;
}

double prob_p8(int geno1, int geno2, double p)
{
  double prob8=0;
  if(geno1==0 && geno2==0)
    prob8=(1-p)*(1-p)*(1-p);
 
  if(geno1==0 && geno2==1)
    prob8=p*(1-p)*(1-p);

  if(geno1==1 && geno2==0)
    prob8=p*(1-p)*(1-p);

  if(geno1==1 && geno2==1)
    prob8=p*(1-p);

  if(geno1==1 && geno2==2)
    prob8=p*p*(1-p);

  if(geno1==2 && geno2==1)
    prob8=p*p*(1-p);

  if(geno1==2 && geno2==2)
    prob8=p*p*p;

  return prob8;
}

double prob_p9(int geno1, int geno2, double p)
{
  double prob9=0;
  if(geno1==0 && geno2==0)
    prob9=(1-p)*(1-p)*(1-p)*(1-p);
 
  if(geno1==0 && geno2==1)
    prob9=2*p*(1-p)*(1-p)*(1-p);

  if(geno1==0 && geno2==2)
    prob9=p*p*(1-p)*(1-p);

 if(geno1==1 && geno2==0)
   prob9=2*p*(1-p)*(1-p)*(1-p);

 if(geno1==1 && geno2==1)
   prob9=4*p*p*(1-p)*(1-p);

 if(geno1==1 && geno2==2)
    prob9=2*p*p*p*(1-p);

  if(geno1==2 && geno2==0)
    prob9=p*p*(1-p)*(1-p);

  if(geno1==2 && geno2==1)
    prob9=2*p*p*p*(1-p);

  if(geno1==2 && geno2==2)
    prob9=p*p*p*p;

  return prob9;
}

int em_algorithm(FILE *outfile, char *tmpid1, char *tmpid2, int n_mrk, double *p1, double *p2, double *p3, double *p4, double *p5, double *p6, double *p7, double *p8, double *p9)
{
  double delta1,delta2,delta3,delta4,delta5,delta6,delta7,delta8,delta9;
  double d_pre1,d_pre2,d_pre3,d_pre4,d_pre5,d_pre6,d_pre7,d_pre8,d_pre9;
  double x1=0,x2=0,x3=0,x4=0,x5=0,x6=0,x7=0,x8=0,x9=0;
  double deno=1;
  double diff=1;
  int m,n_missing;

  delta1=0.1;
  delta2=0.1;
  delta3=0.1;
  delta4=0.1;
  delta5=0.1;
  delta6=0.1;
  delta7=0.1;
  delta8=0.1;
  delta9=0.2;

  while(diff>0.0001){

    x1=0;
    x2=0;
    x3=0;
    x4=0;
    x5=0;
    x6=0;
    x7=0;
    x8=0;
    x9=0;

    n_missing = 0;
    for(m=1;m<=n_mrk;m++){
      if(p1[(m-1)]==0 && p2[(m-1)]==0 && p3[(m-1)]==0 && p4[(m-1)]==0 && p5[(m-1)]==0 && p6[(m-1)]==0 && p7[(m-1)]==0 && p8[(m-1)]==0 && p9[(m-1)]==0){

	n_missing += 1;

      }else{	deno=delta1*p1[(m-1)]+delta2*p2[(m-1)]+delta3*p3[(m-1)]+delta4*p4[(m-1)]+delta5*p5[(m-1)]+delta6*p6[(m-1)]+delta7*p7[(m-1)]+delta8*p8[(m-1)]+delta9*p9[(m-1)];
   
	x1 =  x1 + delta1*p1[(m-1)]/deno;
	x2 =  x2 + delta2*p2[(m-1)]/deno;
	x3 =  x3 + delta3*p3[(m-1)]/deno;
	x4 =  x4 + delta4*p4[(m-1)]/deno;
	x5 =  x5 + delta5*p5[(m-1)]/deno;
	x6 =  x6 + delta6*p6[(m-1)]/deno;
	x7 =  x7 + delta7*p7[(m-1)]/deno;
	x8 =  x8 + delta8*p8[(m-1)]/deno;
	x9 =  x9 + delta9*p9[(m-1)]/deno;
      }
    }

    d_pre1=delta1;
    d_pre2=delta2;
    d_pre3=delta3;
    d_pre4=delta4;
    d_pre5=delta5;
    d_pre6=delta6;
    d_pre7=delta7;
    d_pre8=delta8;
    d_pre9=delta9;
    
    delta1 = (double)x1/(n_mrk-n_missing);
    delta2 = (double)x2/(n_mrk-n_missing);
    delta3 = (double)x3/(n_mrk-n_missing);
    delta4 = (double)x4/(n_mrk-n_missing);
    delta5 = (double)x5/(n_mrk-n_missing);
    delta6 = (double)x6/(n_mrk-n_missing);
    delta7 = (double)x7/(n_mrk-n_missing);
    delta8 = (double)x8/(n_mrk-n_missing);
    delta9 = (double)x9/(n_mrk-n_missing);
   
    diff=sqrt((delta1-d_pre1)*(delta1-d_pre1)+(delta2-d_pre2)*(delta2-d_pre2)+(delta3-d_pre3)*(delta3-d_pre3)+(delta4-d_pre4)*(delta4-d_pre4)+(delta5-d_pre5)*(delta5-d_pre5)+(delta6-d_pre6)*(delta6-d_pre6)+(delta7-d_pre7)*(delta7-d_pre7)+(delta8-d_pre8)*(delta8-d_pre8)+(delta9-d_pre9)*(delta9-d_pre9));

    //    printf("%f %f %f %f %f %f %f %f %f\n",delta1,delta2,delta3,delta4,delta5,delta6,delta7,delta8,delta9);
  }

  fprintf(outfile,"%s %s %f %f %f %f %f %f %f %f %f\n", tmpid1, tmpid2, delta1,delta2,delta3,delta4,delta5,delta6,delta7,delta8,delta9);
  return 0;
}

