//
// Yoonha Choi
// Date: June 8, 2009
//

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

double prob_p0(int geno1, int geno2, double p);
double prob_p1(int geno1, int geno2, double p);
double prob_p2(int geno1, int geno2, double p);

int main(int argc,char *argv[])
{
  if(argc != 5){
    printf("USAGE:./kstat.o <input_file> <output_file> <#individuals> <#markers>\n");
    exit(1);
  }
  
  FILE *infile, *outfile; 
  int n_mrk, n_ind,m;
  double *freq;
  double *p0,*p1,*p2;

  char tmpid1[20],tmpid2[20]; //ID of a pair of individuals
  int *geno1,*geno2;          //Genotype of a pair of individuals
  int n_id1,n_id2;

  n_ind = atoi(argv[3]);
  n_mrk = atoi(argv[4]);

  if(!(infile=fopen(argv[1],"r")))
    puts("Error opening input file\n");

  if((outfile=fopen(argv[2],"w"))==NULL)
    printf("Error creating output file\n");
    
  fprintf(outfile,"ID1 ID2 k0 k1 k2\n");

  // Allele Frequency  
  freq = (double *)malloc((size_t)(n_mrk)*sizeof(double));
  allele_freq(n_ind,n_mrk,infile,freq);
  printf("allele freq done !\n");
  //   for(m=1;m<=n_mrk;m++){
  //    printf("%f ",freq[(m-1)]);
  //  }

  // Compute Pr(Si|delta=j), j=0,1,2

  p0 = (double *)malloc((size_t)(n_mrk)*sizeof(double));
  p1 = (double *)malloc((size_t)(n_mrk)*sizeof(double));
  p2 = (double *)malloc((size_t)(n_mrk)*sizeof(double));
 
  for(m=1;m<=n_mrk;m++){
    p0[(m-1)]=0;
    p1[(m-1)]=0;
    p2[(m-1)]=0;
  }

  geno1 = (int *)malloc((size_t)(n_mrk)*sizeof(int));
  geno2 = (int *)malloc((size_t)(n_mrk)*sizeof(int));

  for(n_id1 = 1 ; n_id1 <= (n_ind-1); n_id1++){
   
    // Skip the individuals before id1 
    for(n_id2 = 1; n_id2<= n_id1 ; n_id2++){
      fscanf(infile,"%s\t",tmpid1);
      for(m=1 ; m <= n_mrk ; m++){
	fscanf(infile,"%d",&geno1[(m-1)]);
      }
    }

    // Input the individual ID2
    while(fscanf(infile,"%s\t",tmpid2)==1){  
      for(m=1 ; m <= n_mrk ; m++){
	fscanf(infile,"%d",&geno2[(m-1)]);
	p0[(m-1)]=(double)prob_p0(geno1[(m-1)],geno2[(m-1)],freq[(m-1)]);
	p1[(m-1)]=(double)prob_p1(geno1[(m-1)],geno2[(m-1)],freq[(m-1)]);
	p2[(m-1)]=(double)prob_p2(geno1[(m-1)],geno2[(m-1)],freq[(m-1)]);
      }
      em_algorithm(outfile,tmpid1,tmpid2,n_mrk,p0,p1,p2);
    }
    rewind(infile);    
  }

  close(infile);
  close(outfile);
}


/* Function for Estimating Allele Frequencies */
int allele_freq(int n_ind, int n_mrk, FILE *infile, double *freq)
{
  char tmpid[20];
  int m, allele;
  int *numerator, *denominator;
  double tmp_double;

  numerator = (int *)malloc((size_t)(n_mrk)*sizeof(int));
  denominator = (int *)malloc((size_t)(n_mrk)*sizeof(int));
  
  for(m=1; m<=n_mrk;m++){
    numerator[(m-1)]=0;
    denominator[(m-1)]=n_ind*2;
  }

  while(fscanf(infile,"%s\t",tmpid)==1){
    for(m=1 ; m <= n_mrk ; m++){
      fscanf(infile,"%d",&allele);

      if(allele == -1){
	denominator[(m-1)] -= 2;
      }else{
	numerator[(m-1)] += allele;
      }
    }
  }
  
  for(m=1; m <= n_mrk ; m++){
    tmp_double= (double)numerator[(m-1)]/denominator[(m-1)];
    freq[(m-1)] = (double)tmp_double;
    //    printf("%d %d %f\n",numerator[(m-1)],denominator[(m-1)],freq[(m-1)]);
  }

  rewind(infile);
  return 0;
}

double prob_p0(int geno1, int geno2, double p)
{
  double prob0=0;
  if(geno1==0 && geno2==0)
    prob0=(1-p)*(1-p)*(1-p)*(1-p);
 
  if(geno1==0 && geno2==1)
    prob0=2*p*(1-p)*(1-p)*(1-p);

  if(geno1==0 && geno2==2)
    prob0=p*p*(1-p)*(1-p);

 if(geno1==1 && geno2==0)
   prob0=2*p*(1-p)*(1-p)*(1-p);

 if(geno1==1 && geno2==1)
   prob0=4*p*p*(1-p)*(1-p);

 if(geno1==1 && geno2==2)
    prob0=2*p*p*p*(1-p);

  if(geno1==2 && geno2==0)
    prob0=p*p*(1-p)*(1-p);

  if(geno1==2 && geno2==1)
    prob0=2*p*p*p*(1-p);

  if(geno1==2 && geno2==2)
    prob0=p*p*p*p;

  return prob0;
}

double prob_p1(int geno1, int geno2, double p)
{
  double prob1=0;
  if(geno1==0 && geno2==0)
    prob1=(1-p)*(1-p)*(1-p);
 
  if(geno1==0 && geno2==1)
    prob1=p*(1-p)*(1-p);

  if(geno1==1 && geno2==0)
    prob1=p*(1-p)*(1-p);

  if(geno1==1 && geno2==1)
    prob1=p*(1-p);

  if(geno1==1 && geno2==2)
    prob1=p*p*(1-p);

  if(geno1==2 && geno2==1)
    prob1=p*p*(1-p);

  if(geno1==2 && geno2==2)
    prob1=p*p*p;

  return prob1;
}

double prob_p2(int geno1, int geno2, double p)
{
  double prob2=0;
  if(geno1==0 && geno2==0)
    prob2=(1-p)*(1-p);
 
  if(geno1==1 && geno2==1)
    prob2=2*p*(1-p);

  if(geno1==2 && geno2==2)
    prob2=p*p;

  return prob2;
}


int em_algorithm(FILE *outfile, char *tmpid1, char *tmpid2, int n_mrk, double *p0, double *p1, double *p2)
{
  double kstat0,kstat1,kstat2;
  double k_pre0,k_pre1,k_pre2;
  double x0=0,x1=0,x2=0;
  double deno=1;
  double diff=1;
  int m,n_missing;

  kstat0=0.4;
  kstat1=0.3;
  kstat2=0.3;

  while(diff>0.0001){

    x0=0;
    x1=0;
    x2=0;
    
    n_missing = 0;
    for(m=1;m<=n_mrk;m++){
      if(p0[(m-1)]==0 && p1[(m-1)]==0 && p2[(m-1)]==0 ){
	n_missing += 1;
      }else{
	deno=kstat0*p0[(m-1)]+kstat1*p1[(m-1)]+kstat2*p2[(m-1)];
	x0 =  x0 + kstat0*p0[(m-1)]/deno;
	x1 =  x1 + kstat1*p1[(m-1)]/deno;
	x2 =  x2 + kstat2*p2[(m-1)]/deno;
      }
    }

    k_pre0= kstat0;
    k_pre1= kstat1;
    k_pre2= kstat2;
    
    kstat0 = (double)x0/(n_mrk-n_missing);
    kstat1 = (double)x1/(n_mrk-n_missing);
    kstat2 = (double)x2/(n_mrk-n_missing);
      
    diff=sqrt((kstat0-k_pre0)*(kstat0-k_pre0)+(kstat1-k_pre1)*(kstat1-k_pre1)+(kstat2-k_pre2)*(kstat2-k_pre2));

  }

  fprintf(outfile,"%s %s %f %f %f\n", tmpid1, tmpid2, kstat0,kstat1,kstat2);
  return 0;
}

